#ifndef __LRU_LIST__
#define __LRU_LIST__

#include <vector>
#include <list>
#include <ostream>

#ifndef __CORBA_H_EXTERNAL_GUARD__
#include <omniORB3/CORBA.h>
#endif

#pragma warning(disable : 4786)  // identifier was truncated to '255' characters in the debug information

class LRUList
{
   public :

      LRUList(
         const size_t maxServants,
         const size_t timeout);

      ~LRUList();

      PortableServer::ObjectId_var Add(PortableServer::RefCountServantBase *pNewObject);

      void Remove(const PortableServer::ObjectId &oid);

      PortableServer::RefCountServantBase *operator[] (const PortableServer::ObjectId &oid);

      size_t GetTimeout() const;

   private :

      class UsageMonitor;

      bool TimedOut(const size_t index) const;

      void Evict();

      PortableServer::RefCountServantBase *UseServant(const size_t index);

      void DisplayList() const;

      size_t OidToIndex(const PortableServer::ObjectId &oid) const;

      PortableServer::ObjectId_var GenerateOid(const size_t index) const;

      size_t m_numServants;
      const size_t m_maxServants;
      const size_t m_timeout;

      std::vector<UsageMonitor> m_servants;
      std::list<size_t> m_freeList;
      std::list<size_t> m_lruList;
};


#endif // __LRU_LIST__